function tBounds=timesegma3(OscRecord,iUgs,iId,ugson=[],shouldPlot=false)

# tBounds = timesegma3(OscRecord, iUgs, iId, ugson, shouldPlot)
#
# Divides a gate charge measurment process into segments corresponding to the 
# different charge components, and returns segment boundaries in tBounds. The 
# structure OscRecord should contain oscilloscope waveforms of gate-source 
# voltage (a waveform with the index iUgs) and drain current (a waveform with 
# the index iId).
#
# The optional parameter ugson specifies the final gate-source voltage value for 
# the determination of the excess charge segment. If it is not provided, the 
# final voltage is equal to the steady-state gate-source voltage with a margin
# of 5%.
#
# The boundaries of the segment corresponding to the QGS2 charge are determined 
# based on the criterion of 5% and 95% of the amplitude of the drain current. 
# The remaining boundaries are determined based on the derivative of the gate-
# source voltage.
#
# If shouldPlot is given and is true, the time boundaries determined are 
# presented in a check plot of the gate-source voltage and the drain current.

# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology
  
  DUGS_FLT_LENGTH = 0.01;
  DUGS_THR = 0.5;
  ID_THR_TH = 0.02;
  ID_THR_ON = 0.05;
  UGSON_THR = 0.05;

  [t,ugs] = oscgetxy(OscRecord,iUgs);
  [t,id] = oscgetxy(OscRecord,iId);
  dugs = diff(ugs);
  dugs = wavglinn(dugs, round(length(dugs)*DUGS_FLT_LENGTH));
  dugs = dugs .* (dugs>0);  
  
  iBounds(1) = searchy(dugs, max(dugs)*DUGS_THR);
  iBounds(2) = searchy(id, max(id)*ID_THR_TH, iBounds(1));
  iBounds(3) = searchy(id, max(id)*(1-ID_THR_ON), iBounds(2));
  iTemp = searchy(dugs, max(dugs)*0.1, iBounds(3));
  iBounds(4) = searchy(dugs, max(dugs(iTemp:end))*DUGS_THR, iTemp);  
  if (isempty(ugson))
    ugson = max(ugs)*(1-UGSON_THR);
  endif
  iBounds(5) = searchy(ugs, ugson, iBounds(4));
  
  tBounds = t(iBounds);

  if (shouldPlot)
    figure();
    subplot(2,1,1);
    plot(t, ugs);
    hold on;
    for it = [1:length(tBounds)]
      plot(tBounds(it)*[1 1], [0 max(ugs)], ":b");
    endfor
    grid on;
    xlabel("t /s");
    ylabel("uGS / V");
    subplot(2,1,2);
    plot(t, id);
    hold on;
    for it = [2:3]
      plot(tBounds(it)*[1 1], [0 max(id)], ":b");
    endfor
    grid on;
    xlabel("t /s");
    ylabel("iD / A");
  endif
  
endfunction
